////////////////////////////////////////////////////////////////////////////////
// class   KrDFree
// Version 2.0
// Date    26-12-1997
// E-Mail  kray@iaehv.nl
// Page    www.iaehv.nl/users/kray
//
// 5 sept 1998  For larger partitions as 2 gb
//              Works only with delphi 4.0 and later
////////////////////////////////////////////////////////////////////////////////

unit Kr_DFree;
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, extctrls,
  dsgnIntf;

type
  KrDFree=class;
  KrAbout=class;

  TSize=(siByte, siKB, siMeg);
  TBorder=(boNone, bo3dUp, bo3dDown, boLine);
  TFigure=(fiBar, fiText, fiPercent);
  TCapAlign=(alLeft, alRight, alCenter, alBottom, alTop);
  TCapType=(caNone, caCaption, caDriveLetter, caPerc, caDriveType);

  TBeforeChange=procedure(Sender:TObject) of object;
  TAfterChange=procedure(Sender:TObject) of object;
  TBeforeFirstTime=procedure(Sender:TObject) of object;
  TBeforePaint=procedure(Sender:TObject; i:integer) of Object;
  TAfterPaint=procedure(Sender:TObject; i:integer) of Object;
  TAlarm=procedure(Sender:TObject) of object;

  KrGeg=Record
    Found:boolean;
    Size:int64;
    Free:int64;
    DriveType:word;
    MessageOnce:boolean;
  end;

  KrAlarm=class(TPersistent)
  private
    _Alarm:boolean;
    _Color:TColor;
    _Percent:integer;
    _MessageOnce:boolean;
  public
    constructor Create;
  published
    property Alarm:boolean read _Alarm write _Alarm default True;
    property Color:TColor read _Color write _Color default clRed;
    property Percent:integer read _Percent write _Percent default 10;
    property MessageOnce:boolean read _MessageOnce write _MessageOnce default True;
  end;

  KrDrive=class(TPersistent)
  private
    _AutoDetect:boolean;
    _Drives:string;
    _ExcludeDrive:string;
    _ExcludeRo:boolean;
  public
    DFree:KrDFree;
    Geg:array[1..26] of KrGeg;

    constructor Create;
    procedure SetAutoDetect(value:boolean);
    procedure SetDrives(value:string);
    procedure SetExcludeRo(value:boolean);

    procedure FindDrives;
  published
    property AutoDetect:boolean read _AutoDetect write SetAutoDetect;
    property Drives:string read _Drives write SetDrives;
    property ExcludeDrive:string read _ExcludeDrive write _ExcludeDrive;
    property ExcludeRo:boolean read _ExcludeRo write SetExcludeRo default True;            
  end;

  KrBar=class(TPersistent)
  private
    _BackGround:TColor;
    _Border:TBorder;
    _LineColor:TColor;
    _LineColorDown:TColor;
    _LineColorUp:TColor;
    _Color:TColor;

    DFree:KrDfree;
    Bar:TRect;
  public
    constructor Create;
    procedure SetBackGround(value:TColor);
    procedure SetBorder(value:TBorder);
    procedure SetLineColor(value:TColor);
    procedure SetLineColorDown(value:TColor);
    procedure SetLineColorUp(value:TColor);
    procedure SetColor(value:TColor);
  published
    property Color:TColor read _Color write _Color default clGreen;
    property BackGround:TColor read _BackGround write SetBackGround default clSilver;
    property Border:TBorder read _Border write SetBorder default bo3dUp;
    property LineColor:TColor read _LineColor write SetLineColor default clBlack;
    property LineColorUp:TColor read _LineColorUp write SetLineColorUp default clWhite;
    property LineColorDown:TColor read _LineColorDown write SetLineColorDown default clGray;
  end;

  KrBorder=class(TPersistent)
  private
    _BackGround:TColor;
    _Border:TBorder;
    _LineColor:TColor;
    _LineColorDown:TColor;
    _LineColorUp:TColor;
    _Transparant:boolean;

    DFree:KrDFree;
  public
    constructor Create;
    procedure SetBackGround(value:TColor);
    procedure SetBorder(value:TBorder);
    procedure SetLineColor(value:TColor);
    procedure SetLineColorUp(value:TColor);
    procedure SetLineColorDown(value:TColor);
    procedure SetTransparant(value:boolean);
  published
    property BackGround:TColor read _BackGround write SetBackGround default clSilver;
    property Border:TBorder read _Border write SetBorder default bo3dUp;
    property LineColor:TColor read _LineColor write SetLineColor default clBlack;
    property LineColorUp:TColor read _LineColorUp write SetLineColorUp default clWhite;
    property LineColorDown:TColor read _LineColorDown write SetLineColorDown default clGray;
    property Transparant:boolean read _Transparant write SetTransparant default False;
  end;

  KrDFree = class(TGraphicControl)
  private
    _About:KrAbout;
    _Alarm:KrAlarm;
    _Align:TCapAlign;
    _Bar:KrBar;
    _Border:KrBorder;
    _Caption:string;
    _CaptionType:TCapType;
    _CurrentFree:int64;
    _CurrentSize:int64;
    _CurrentDriveType:string;
    _CurrentDriveLetter:string;
    _Digits:integer;
    _Drive:KrDrive;
    _Figure:TFigure;
    _Interval:integer;
    _Margin:integer;
    _Reset:boolean;
    _Size:TSize;
    _TextWidth:integer;

    _BeforeFirstTime:TBeforeFirstTime;
    _BeforePaint:TBeforePaint;
    _AfterPaint:TAfterPaint;
    _OnAlarm:TAlarm;

    _Timer:TTimer;

    TextHeight:integer;
    TextWidth:integer;
    CurrentOffset:integer;
    TotDrives:integer;
    OrgHeight:integer;
    BarRect:TRect;

    procedure SetAlign(value:TCapAlign);
    procedure SetCaption(value:string);
    procedure SetCaptionType(value:TCapType);
    procedure SetDigits(value:integer);
    procedure SetFigure(value:TFigure);
    procedure SetMargin(value:integer);
    procedure SetInterval(value:integer);
    procedure SetReset(value:boolean);
    procedure SetSize(value:TSize);
    procedure SetTextWidth(value:integer);

    procedure TimerExpired(Sender:Tobject);

    procedure PaintBorder;
    procedure PaintBorderDown;
    procedure PaintBorderLine;
    procedure PaintBorderNone;
    procedure PaintBorderUp;

    procedure GetTextRanges;
    procedure PaintText;

    procedure PaintBarBorder;
    procedure PaintBarBorderLine(l:boolean);
    procedure PaintBarBorder3d(b:boolean);
    procedure PaintBar;
    procedure PaintPercent;
    procedure PaintFiText;
    procedure PaintFigure;

    procedure CM_PARENTCOLORCHANGED(var Msg:TMessage); message cm_ParentColorChanged;
  protected
    procedure Paint; override;

    procedure DoBeforeFirstTime; virtual;
    procedure DoBeforePaint(i:integer); virtual;
    procedure DoAfterPaint(i:integer); virtual;
    procedure DoAlarm; virtual;
  public
    constructor Create(AOwner:TComponent); override;
    destructor Destroy; override;

    procedure GetFree;
    procedure Init;
  published
    property About:KrAbout read _About write _About;
    property Alarm:KrAlarm read _Alarm write _Alarm;
    property AlignText:TCapAlign read _Align write SetAlign default alLeft;
    property Bar:KrBar read _Bar write _Bar;
    property Border:KrBorder read _Border write _Border;
    property Caption:string read _Caption write SetCaption;
    property CaptionType:TCapType read _CaptionType write SetCaptionType default caCaption;
    property CurrentDriveLetter:string read _CurrentDriveLetter write _CurrentDriveLetter;
    property CurrentDriveType:string read _CurrentDriveType write _CurrentDriveType;
    property CurrentFree:int64 read _CurrentFree write _CurrentFree;
    property CurrentSize:int64 read _CurrentSize write _CurrentSize;
    property DefaultTextWidth:integer read _TextWidth write SetTextWidth;
    property Digits:integer read _Digits write SetDigits default 2;
    property Drive:KrDrive read _Drive write _Drive;
    property Figure:TFigure read _Figure write SetFigure default fiBar;
    property Interval:integer read _Interval write SetInterval default 1000;
    property Margin:integer read _Margin write SetMargin default 5;
    property Reset:boolean read _Reset write SetReset;
    property Size:TSize read _Size write SetSize default siByte;

    property ParentShowHint;
    property ShowHint;
    property Hint;
    property Font;
    property Tag;
    property Enabled;

    property OnBeforeFirstTime:TBeforeFirstTime read _BeforeFirstTime write _BeforeFirstTime;
    property OnBeforePaint:TBeforePaint read _BeforePaint write _BeforePaint;
    property OnAfterPaint:TAfterPaint read _AfterPaint write _AfterPaint;
    property OnAlarm:TAlarm read _OnAlarm write _OnAlarm;

    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
  end;

  KrAbout=class(TPropertyEditor)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
    function GetValue: string; override;
  end;

procedure Register;

implementation

///////////////////////////////////////////////////////////////////////////////
procedure Register;
  begin
    RegisterComponents('Kr...', [KrDFree]);
    RegisterPropertyEditor(TypeInfo(KrAbout), KrDFree, 'About', KrAbout);
  end;
///////////////////////////////////////////////////////////////////////////////
  constructor KrAlarm.Create;
  begin
    inherited Create;
    _Color:=clRed;
    _Alarm:=True;
    _Percent:=10;
    _MessageOnce:=True;
  end;
///////////////////////////////////////////////////////////////////////////////
  constructor KrBar.Create;
  begin
    inherited Create;
    _Color:=clGreen;
    _BackGround:=clSilver;
    _Border:=bo3dUp;
    _LineColor:=clBlack;
    _LineColorDown:=clGray;
    _LineColorUp:=clWhite;
  end;

  procedure KrBar.SetColor(value:TColor);
  begin
    _Color:=value;
    if csDesigning in DFree.ComponentState then DFree.Paint;
  end;

  procedure KrBar.SetBackGround(value:TColor);
  begin
    _BackGround:=value;
    if csDesigning in DFree.ComponentState then DFree.Paint;
  end;

  procedure KrBar.SetLineColor(value:TColor);
  begin
    _LineColor:=value;
    if csDesigning in Dfree.ComponentState then DFree.Paint;
  end;

  procedure KrBar.SetLineColorDown(value:TColor);
  begin
    _LineColorDown:=value;
    if csDesigning in DFree.ComponentState then DFree.Paint;
  end;

  procedure KrBar.SetLineColorUp(value:TColor);
  begin
    _LineColorUp:=value;
    if csDesigning in DFree.ComponentState then DFree.Paint;
  end;

  procedure KrBar.SetBorder(value:TBorder);
  begin
    _Border:=Value;
    if csDesigning in DFree.ComponentState then DFree.Paint;
  end;
///////////////////////////////////////////////////////////////////////////////
  constructor KrDrive.Create;
  begin
    inherited Create;
    _ExcludeRo:=True;
  end;

  procedure KrDrive.SetAutoDetect(value:boolean);
  begin
    _AutoDetect:=value;
    if csDesigning in DFree.ComponentState then begin
      DFree.Init;
      if _AutoDetect=True then FindDrives;
      DFree.Paint;
    end;
  end;

  procedure KrDrive.SetDrives(value:string);
  begin
    _Drives:=value;
    if csDesigning in DFree.ComponentState then begin
      DFree.Init;
      if length(_Drives)>0 then FindDrives;
      DFree.Paint;
    end;
  end;

  procedure KrDrive.SetExcludeRo(value:boolean);
  begin
    _ExcludeRO:=value;
    if csDesigning in DFree.ComponentState then DFree.Paint;
  end;

  procedure KrDrive.FindDrives;
  var
    i:integer;
    s:int64;
    d, e:string;
    k:string;
  begin
    if DFree.OrgHeight<>0 then DFree.Height:=DFree.OrgHeight;
    d:=UpperCase(_Drives);
    e:=Uppercase(_ExcludeDrive);
    DFree.TotDrives:=0;
    for i:=1 to 26 do begin
      if pos(chr(i+64), e)=0 then begin
        if (_AutoDetect=True) or ((_AutoDetect=False) and (pos(chr(i+64), d)>0)) then begin
          s:=DiskSize(i);
          if s>0 then begin
            k:=chr(i+64)+':';
            if (_ExcludeRo=False) or ((_ExcludeRo=True) and (GetDriveType(pchar(k))<>DRIVE_CDROM)) then begin
              Geg[i].Found:=True;
              Geg[i].Size:=s;
              Geg[i].Free:=DiskFree(i);
              Geg[i].MessageOnce:=False;
              Geg[i].DriveType:=GetDriveType(pchar(k));
              inc(DFree.TotDrives);
            end;
          end;
        end;
      end;
    end;
    DFree.OrgHeight:=DFree.Height;
    if not(csDesigning in DFree.ComponentState) then DFree.Height:=DFree.OrgHeight*DFree.TotDrives;
  end;
///////////////////////////////////////////////////////////////////////////////
  constructor KrBorder.Create;
  begin
    inherited Create;
    _BackGround:=clSilver;
    _Border:=bo3dUp;
    _LineColor:=clBlack;
    _LineColorDown:=clGray;
    _LineColorUp:=clWhite;
    _TransParant:=False;
  end;

  procedure KrBorder.SetBackGround(value:TColor);
  begin
    _BackGround:=value;
    if csDesigning in DFree.ComponentState then DFree.Paint;
  end;

  procedure KrBorder.SetBorder(value:TBorder);
  begin
    _Border:=value;
    if csDesigning in DFree.ComponentState then DFree.Paint;
  end;

  procedure KrBorder.SetLineColor(value:TColor);
  begin
    _LineColor:=value;
    if csDesigning in DFree.ComponentState then DFree.Paint;
  end;

  procedure KrBorder.SetLineColorDown(value:TColor);
  begin
    _LineColorDown:=value;
    if csDesigning in DFree.ComponentState then DFree.Paint;
  end;

  procedure KrBorder.SetLineColorUp(value:TColor);
  begin
    _LineColorUp:=value;
    if csDesigning in DFree.ComponentState then DFree.Paint;
  end;

  procedure KrBorder.SetTransParant(value:boolean);
  begin
    _TransParant:=value;
    if csDesigning in DFree.ComponentState then DFree.Paint;
  end;
///////////////////////////////////////////////////////////////////////////////
  constructor KrDFree.Create(AOwner:TComponent);
  begin
    inherited Create(AOwner);
    _Alarm:=KrAlarm.Create;

    _Bar:=KrBar.Create;
    _Bar.DFree:=Self;

    _Border:=KrBorder.Create;
    _Border.DFree:=Self;

    _Drive:=KrDrive.Create;
    _Drive.DFree:=Self;

    _Align:=alLeft;
    _Digits:=2;
    _Margin:=5;
    _CaptionType:=caCaption;
    _Figure:=fiBar;
    _Interval:=1000;
    _Size:=siByte;

    if _Interval<>0 then begin
      _Timer:=TTimer.Create(Self);
      _Timer.Interval:=_Interval;
      _Timer.Ontimer:=TimerExpired;
      _Timer.Enabled:=True;
    end;

    _Drive.FindDrives;
  end;

  destructor KrDFree.Destroy;
  begin
    _Drive.Free;
    _Bar.Free;
    _Alarm.Free;
    _Border.Free;
    if _Timer<>Nil then _Timer.Free;
    inherited Destroy;
  end;

  procedure KrDFree.TimerExpired(Sender:TObject);
  begin
    if csDesigning in ComponentState then exit;
    GetFree;
    Paint;
  end;

  procedure KrDFree.CM_PARENTCOLORCHANGED(var msg:TMessage);
  begin
    inherited;
    Paint;
  end;

  procedure KrDFree.Paint;
  var
    i:integer;
  begin
    if (csDesigning in ComponentState) then OrgHeight:=Height;
    DoBeforeFirstTime;
    PaintBorder;
    CurrentOffset:=0;

    for i:=1 to 26 do begin
      if _Drive.Geg[i].Found=True then begin
        inc(CurrentOffset);
        case _Drive.Geg[i].DriveType of
          DRIVE_REMOVABLE: _CurrentDriveType:='Removable';
          DRIVE_FIXED:     _CurrentDriveType:='Fixed';
          DRIVE_REMOTE:    _CurrentDriveType:='Remote';
          DRIVE_CDROM:     _CurrentDriveType:='CdRom';
          DRIVE_RAMDISK:   _CurrentDriveType:='RamDisk';
        end;
        _CurrentSize:=_Drive.Geg[i].Size;
        _CurrentFree:=_Drive.Geg[i].Free;
        _CurrentDriveLetter:=chr(i+64);
        DoBeforePaint(CurrentOffset);

        GetTextRanges;
        PaintText;
        PaintBarBorder;
        PaintFigure;

        DoAfterPaint(CurrentOffset);
        if (csDesigning in ComponentState) and (CurrentOffset>0) then exit;
      end;
    end;
  end;

  procedure KrDFree.SetAlign(value:TCapAlign);
  begin
    _Align:=value;
    if csDesigning in ComponentState then Paint;
  end;

  procedure KrDFree.SetDigits(value:integer);
  begin
    if value<6 then _Digits:=value;
    if csDesigning in ComponentState then Paint;
  end;

  procedure KrDFree.SetCaption(value:string);
  begin
    _Caption:=value;
    if csDesigning in ComponentState then Paint;
  end;

  procedure KrDFree.SetCaptionType(value:TCapType);
  begin
    _CaptionType:=value;
    if csDesigning in ComponentState then Paint;
  end;

  procedure KrDFree.SetFigure(value:TFigure);
  begin
    _Figure:=value;
    if csDesigning in ComponentState then Paint;
  end;

  procedure KrDFree.SetMargin(value:integer);
  begin
    _Margin:=value;
    if csDesigning in ComponentState then Paint;
  end;

  procedure KrDFree.SetReset(value:boolean);
  begin
    Init;
    _Drive.FindDrives;
    Paint;
    _Reset:=False;
  end;

  procedure KrDFree.SetInterval(value:integer);
  begin
    _Interval:=value;
    if _Timer<>Nil then _Timer.Interval:=_Interval;
  end;

  procedure KrDFree.SetSize(value:TSize);
  begin
    _Size:=value;
    if csDesigning in ComponentState then Paint;
  end;

  procedure KrDFree.SetTextWidth(value:integer);
  begin
    _TextWidth:=value;
    if csDesigning in ComponentState then Paint;
  end;
  //////////////////////////////////////////////////////////////////////////////
  procedure KrDFree.DoBeforeFirstTime;
  begin
    if Assigned(_BeforeFirstTime) then _BeforeFirstTime(Self);
  end;

  procedure KrDFree.DoBeforePaint(i:integer);
  begin
    if Assigned(_BeforePaint) then _BeforePaint(Self, i);
  end;

  procedure KrDFree.DoAfterPaint(i:integer);
  begin
    if Assigned(_AfterPaint) then _AfterPaint(Self, i);
  end;

  procedure KrDFree.DoAlarm;
  begin
    if Assigned(_OnAlarm) then _OnAlarm(Self);
  end;
  /////////////////////////////////////////////////////////////////////////////
  procedure KrDFree.GetFree;
  var
    i:integer;
  begin
    for i:=1 to 26 do
      if _Drive.Geg[i].Found=True then _Drive.Geg[i].Free:=DiskFree(i);
  end;

  procedure KrDFree.GetTextRanges;
  var
    s:string;
    d:double;
  begin
    case CaptionType of
      caNone:        s:='';
      caCaption:     s:=_Caption;
      caDriveLetter: s:=_CurrentDriveLetter+':';
      caDriveType:   s:=_CurrentDriveLetter+': '+_CurrentDriveType;
      caPerc:        begin
                       d:=_CurrentFree;
                       d:=(d*100)/_CurrentSize+integer(_CurrentSize=0);
                       Str(d:6:_Digits, s);
                       s:=s+' %';
                       s:=_CurrentDriveLetter+': '+s;
                     end;
    end;
    TextHeight:=canvas.TextHeight(s);
    if (_TextWidth=0) or (_Align=alCenter) or (_Align=alTop) or (_Align=alBottom) then
      TextWidth:=canvas.TextWidth(s)
    else
      TextWidth:=_TextWidth;
  end;

  procedure KrDFree.Init;
  var
    i:integer;
  begin
    DoBeforeFirstTime;
    for i:=1 to 26 do begin
      _Drive.Geg[i].Found:=False;
      _Drive.Geg[i].Size:=0;
      _Drive.Geg[i].Free:=0;
      _Drive.Geg[i].DriveType:=0;
      _Drive.Geg[i].MessageOnce:=False;
    end;
  end;

  procedure KrDFree.PaintBarBorder;
  begin
    case _Bar._Border of
      boNone:   PaintBarBorderLine(False);
      bo3dUp:   PaintBarBorder3D(True);
      bo3dDown: PaintBarBorder3d(False);
      boLine:   PaintBarBorderLine(True);
    end;
  end;

  procedure KrDFree.PaintBarBorder3d(b:boolean);
  begin
    PaintBarBorderLine(False);
    case _Align of
    alLeft: begin
        if b=True then Canvas.Pen.Color:=_Bar._LineColorUp else Canvas.Pen.Color:=_Bar._LineColorDown;
        Canvas.MoveTo(TextWidth+_Margin*2, CurrentOffset*OrgHeight-_Margin);
        Canvas.LineTo(TextWidth+_Margin*2, (CurrentOffset-1)*OrgHeight+_Margin);
        Canvas.LineTo(Width-_Margin, (CurrentOffset-1)*OrgHeight+_Margin);

        if b=True then Canvas.Pen.Color:=_Bar.LineColorDown else Canvas.Pen.Color:=_Bar._LineColorUp;
        Canvas.LineTo(Width-_Margin, CurrentOffset*OrgHeight-_Margin);
        Canvas.LineTo(TextWidth+_Margin*2, CurrentOffset*OrgHeight-_Margin);
      end;
    alRight:begin
        if b=True then Canvas.Pen.Color:=_Bar._LineColorUp else Canvas.Pen.Color:=_Bar._LineColorDown;
        Canvas.MoveTo(_Margin, CurrentOffset*OrgHeight-_Margin);
        Canvas.LineTo(_Margin, (CurrentOffset-1)*OrgHeight+_Margin);
        Canvas.LineTo(Width-TextWidth-_Margin*2, (CurrentOffset-1)*OrgHeight+_Margin);

        if b=True then Canvas.Pen.Color:=_Bar._LineColorDown else Canvas.Pen.Color:=_Bar._LineColorUp;
        Canvas.LineTo(Width-TextWidth-_Margin*2, CurrentOffset*OrgHeight-_Margin);
        Canvas.LineTo(_Margin, CurrentOffset*OrgHeight-_Margin);
      end;
    alCenter:begin
        if b=True then Canvas.Pen.Color:=_Bar._LineColorUp else Canvas.Pen.Color:=_Bar._LineColorDown;
        Canvas.MoveTo(_Margin, CurrentOffset*OrgHeight-_Margin);
        Canvas.LineTo(_Margin, (CurrentOffset-1)*OrgHeight+_Margin);
        Canvas.LineTo(Width-_Margin, (CurrentOffset-1)*OrgHeight+_Margin);

        if b=True then Canvas.Pen.Color:=_Bar._LineColorDown else Canvas.Pen.Color:=_Bar._LineColorUp;
        Canvas.LineTo(Width-_Margin, CurrentOffset*OrgHeight-_Margin);
        Canvas.LineTo(_Margin, CurrentOffset*OrgHeight-_Margin);
      end;
    alBottom:begin
        if b=True then Canvas.Pen.Color:=_Bar._LineColorUp else Canvas.Pen.Color:=_Bar._LineColorDown;
        Canvas.MoveTo(_Margin, CurrentOffset*OrgHeight-(_Margin*2)-TextHeight);
        Canvas.LineTo(_Margin, (CurrentOffset-1)*OrgHeight+_Margin);
        Canvas.LineTo(Width-_Margin, (CurrentOffset-1)*OrgHeight+_Margin);

        if b=True then Canvas.Pen.Color:=_Bar._LineColorDown else Canvas.Pen.Color:=_Bar._LineColorUp;
        Canvas.LineTo(Width-_Margin, CurrentOffset*OrgHeight-(_Margin*2)-TextHeight);
        Canvas.LineTo(_Margin, CurrentOffset*OrgHeight-(_Margin*2)-TextHeight);
      end;
    alTop:begin
        if b=True then Canvas.Pen.Color:=_Bar._LineColorUp else Canvas.Pen.Color:=_Bar._LineColorDown;
        Canvas.MoveTo(_Margin, CurrentOffset*OrgHeight-_Margin);
        Canvas.LineTo(_Margin, (CurrentOffset-1)*OrgHeight+(_Margin*2)+TextHeight);
        Canvas.LineTo(Width-_Margin, (CurrentOffset-1)*OrgHeight+(_Margin*2)+TextHeight);

        if b=True then Canvas.Pen.Color:=_Bar._LineColorDown else Canvas.Pen.Color:=_Bar._LineColorUp;
        Canvas.LineTo(Width-Margin, CurrentOffset*OrgHeight-_Margin);
        Canvas.LineTo(_Margin, CurrentOffset*OrgHeight-_Margin);
      end;
    end;
  end;

  procedure KrDFree.PaintBarBorderLine(l:boolean);
  begin
    if l=True then
      canvas.pen.Color:=_Bar._LineColor
    else
      canvas.Pen.color:=_Bar._BackGround;
    canvas.brush.Color:=_Bar._BackGround;

    case _Align of
    alLeft: begin
        BarRect.Left:=TextWidth+_Margin*2;
        BarRect.Top:=(CurrentOffset-1)*OrgHeight+_Margin;
        BarRect.Right:=Width-_Margin;
        BarRect.Bottom:=CurrentOffset*OrgHeight-_Margin;
      end;
    alRight:begin
        BarRect.Left:=_Margin;
        BarRect.Top:=(CurrentOffset-1)*OrgHeight+_Margin;
        BarRect.Right:=Width-TextWidth-_Margin*2;
        BarRect.Bottom:=CurrentOffset*OrgHeight-_Margin;
      end;
    alCenter:begin
        BarRect.Left:=_Margin;
        BarRect.Top:=(CurrentOffset-1)*OrgHeight+_Margin;
        BarRect.Right:=Width-_Margin;
        BarRect.Bottom:=CurrentOffset*OrgHeight-_Margin;
      end;
    alBottom:begin
        BarRect.Left:=_Margin;
        BarRect.Top:=(CurrentOffset-1)*OrgHeight+_Margin;
        BarRect.Right:=Width-_Margin;
        BarRect.Bottom:=CurrentOffset*OrgHeight-(_Margin*2)-TextHeight;
      end;
    alTop:begin
        BarRect.Left:=_Margin;
        BarRect.Top:=(CurrentOffset-1)*OrgHeight+(_Margin*2)+TextHeight;
        BarRect.Right:=Width-_Margin;
        BarRect.Bottom:=CurrentOffset*OrgHeight-_Margin;
      end;
    end;
    canvas.Rectangle(BarRect.Left, BarRect.Top, BarRect.Right, BarRect.Bottom);
    _Bar.Bar:=BarRect;
  end;

  procedure KrDFree.PaintBorder;
  begin
    case _Border._Border of
    boLine:   PaintBorderLine;
    boNone:   PaintBorderNone;
    bo3dUp:   PaintBorderUp;
    bo3dDown: PaintBorderDown;
    end;
  end;

  procedure KrDFree.PaintBorderLine;
  begin
    with canvas do begin
      pen.Color:=_Border._LineColor;
      if _Border._Transparant=True then
        brush.Color:=TWinControl(Parent).Brush.Color
      else
        brush.Color:=_Border._BackGround;
      Rectangle(0, 0, Width, Height);
    end;
  end;

  procedure KrDFree.PaintBorderNone;
  var
    r:TRect;
  begin
    with canvas do begin
      pen.Color:=_Border._LineColor;
      if _Border._Transparant=True then
        brush.Color:=TWinControl(Parent).Brush.Color
      else
        brush.Color:=_Border._BackGround;
      r.Left:=0;
      r.Top:=0;
      r.Bottom:=Height;
      r.Right:=Width;
      FillRect(r);
    end;
  end;

  procedure KrDFree.PaintBorderUp;
  begin
    PaintBorderNone;
    with canvas do begin
      pen.Color:=_Border._LineColorUp;
      MoveTo(0, Height);
      LineTo(0,0);
      LineTo(Width-1, 0);
      pen.Color:=_Border._LineColorDown;
      LineTo(Width-1, Height-1);
      LineTo(0, Height-1);
    end;
  end;

  procedure KrDFree.PaintBorderDown;
  begin
    PaintBorderNone;
    with canvas do begin
      pen.Color:=_Border._LineColorDown;
      MoveTo(0, Height);
      LineTo(0,0);
      LineTo(Width-1, 0);
      pen.Color:=_Border._LineColorUp;
      LineTo(Width-1, Height-1);
      LineTo(0, Height-1);
    end;
  end;

  procedure KrDFree.PaintBar;
  var
    t:double;
  begin
    inc(_Bar.Bar.Top);
    inc(_Bar.Bar.Left);
    dec(_Bar.Bar.Bottom);
    dec(_Bar.Bar.Right);

    with canvas do begin
      if _Alarm._Alarm=True then begin
        t:=_CurrentFree;
        t:=(t*100)/_CurrentSize;
        if trunc(t)<=_Alarm._Percent then begin
          Pen.Color:=_Alarm._Color;
          Brush.Color:=_Alarm._Color;
          if _Alarm.MessageOnce=False then DoAlarm
          else begin
            if _Drive.Geg[CurrentOffset].MessageOnce=False then DoAlarm;
            _Drive.Geg[CurrentOffset].MessageOnce:=True;
          end;
        end else begin
          Pen.Color:=_Bar._Color;
          Brush.Color:=_Bar._Color;
          _Drive.Geg[CurrentOffset].MessageOnce:=False;
        end;
      end else begin
        Pen.Color:=_Bar._Color;
        Brush.Color:=_Bar._Color;
      end;

      t:=(_CurrentSize-_CurrentFree)/_CurrentSize;
      _Bar.Bar.Right:=_Bar.Bar.Right-_Bar.Bar.Left;
      t:=t*_Bar.Bar.Right+_Bar.Bar.Left;
      _Bar.Bar.Right:=trunc(t);
      FillRect(_Bar.Bar);
    end;
  end;

  procedure KrDFree.PaintFigure;
  begin
    case _Figure of
    fiBar:     PaintBar;
    fiPercent: PaintPercent;
    fiText:    PaintFiText;
    end;
  end;

  procedure KrDFree.PaintFiText;
  var
    f, d:int64;
    s:string;
    th, tw:integer;
    t:double;
  begin
    f:=0;
    d:=0;
    inc(_Bar.Bar.Top);
    inc(_Bar.Bar.Left);
    dec(_Bar.Bar.Bottom);
    dec(_Bar.Bar.Right);

    canvas.Font:=Font;
    if _Alarm._Alarm=True then begin
      t:=_CurrentFree;
      t:=(t*100)/_CurrentSize;
      if trunc(t)<=_Alarm._Percent then begin
        canvas.Font.Color:=_Alarm._Color;
        if _Alarm.MessageOnce=False then DoAlarm
        else begin
          if _Drive.Geg[CurrentOffset].MessageOnce=False then DoAlarm;
          _Drive.Geg[CurrentOffset].MessageOnce:=True;
        end;
      end else begin
        canvas.Font.Color:=_Bar._Color;
        _Drive.Geg[CurrentOffset].MessageOnce:=False;
      end;
    end else
      canvas.Font.Color:=_Bar._Color;

    case _Size of
    siByte: begin
        d:=_CurrentSize;
        f:=_CurrentFree;
        s:=' Bytes';
      end;
    siKB: begin
        d:=_CurrentSize div 1024;
        f:=_CurrentFree div 1024;
        s:=' Kilobytes';
      end;
    siMeg: begin
        d:=_CurrentSize div 1048576;
        f:=_CurrentFree div 1048576;
        s:=' Megabytes';
      end;
    end;

    s:='Size:'+inttostr(d)+'/'+'Free:'+inttostr(f)+s;
    tw:=canvas.TextWidth(s);
    th:=canvas.TextHeight(s);
    canvas.TextRect(_Bar.Bar,
                    (_Bar.Bar.Left+_Bar.Bar.Right) div 2 - tw div 2,
                    (_Bar.Bar.Top+_Bar.Bar.Bottom) div 2 - th div 2, s);
  end;

  procedure KrDFree.PaintPercent;
  var
    t:double;
    s:string;
    th, tw:integer;
  begin
    inc(_Bar.Bar.Top);
    inc(_Bar.Bar.Left);
    dec(_Bar.Bar.Bottom);
    dec(_Bar.Bar.Right);

    canvas.Font:=Font;
    if _Alarm._Alarm=True then begin
      t:=_CurrentFree;
      t:=(t*100)/_CurrentSize;
      if trunc(t)<=_Alarm._Percent then begin
        canvas.Font.Color:=_Alarm._Color;
        if _Alarm.MessageOnce=False then DoAlarm
        else begin
          if _Drive.Geg[CurrentOffset].MessageOnce=False then DoAlarm;
          _Drive.Geg[CurrentOffset].MessageOnce:=True;
        end;
      end else begin
        canvas.Font.Color:=_Bar._Color;
        _Drive.Geg[CurrentOffset].MessageOnce:=False;
      end;
    end else
      canvas.Font.Color:=_Bar._Color;

    t:=_CurrentFree;
    t:=(t*100)/_CurrentSize+integer(_CurrentSize=0);
    Str(t:6:_Digits, s);
    s:=s+' %';
    tw:=canvas.TextWidth(s);
    th:=canvas.TextHeight(s);
    canvas.TextRect(_Bar.Bar,
                    (_Bar.Bar.Left+_Bar.Bar.Right) div 2 - tw div 2,
                    (_Bar.Bar.Top+_Bar.Bar.Bottom) div 2 - th div 2, s);
  end;

  procedure KrDFree.PaintText;
  var
    r:TRect;
    d:double;
    s:string;
  begin
    if (_CaptionType=caNone) or (_Align=alCenter) then exit;

    canvas.Font:=Font;
    if _Border._Transparant=True then begin
      canvas.brush.Color:=TWinControl(Parent).Brush.Color;
      canvas.pen.Color:=TWinControl(Parent).Brush.Color;
    end else begin
      canvas.brush.Color:=_Border._BackGround;
      canvas.pen.Color:=_Border._BackGround;
    end;

    case _CaptionType of
      caCaption:     s:=_Caption;
      caDriveLetter: s:=_CurrentDriveLetter+':';
      caDriveType:   s:=_CurrentDriveLetter+': '+_CurrentDriveType;
      caPerc:        begin
                       d:=_CurrentFree;
                       d:=(d*100)/_CurrentSize+integer(_CurrentSize=0);
                       Str(d:6:_Digits, s);
                       s:=s+' %';
                       s:=_CurrentDriveLetter+': '+s;
                     end;
    end;
    s:=Trim(s);
    case _Align of
      alLeft: begin
        r.Left:=_Margin;
        r.Top:=(CurrentOffset-1)*OrgHeight+1;
        r.Right:=TextWidth+_Margin+1;
        r.Bottom:=r.Top+OrgHeight-2;
        canvas.TextRect(r, r.Left, (r.Top+r.Bottom) div 2 - TextHeight div 2, s);
      end;
      alRight: begin
        r.Left:=Width-_Margin-TextWidth;
        r.Top:=(CurrentOffset-1)*OrgHeight+1;
        r.Right:=Width-1;
        r.Bottom:=r.Top+OrgHeight-2;
        canvas.TextRect(r, r.Left, (r.Top+r.Bottom) div 2 - TextHeight div 2, s);
      end;
      alTop: begin
        r.Left:=1;
        r.Top:=(CurrentOffset-1)*OrgHeight+1+_Margin;
        r.Bottom:=r.Top+OrgHeight-2-_Margin;
        r.Right:=Width-1;
        canvas.TextRect(r,(r.Left+r.Right) div 2 - TextWidth div 2, r.Top, s);
      end;
      alBottom: begin
        r.Left:=1;
        r.Top:=CurrentOffset*OrgHeight-_Margin-TextHeight;
        r.Bottom:=CurrentOffset*OrgHeight-2;
        r.Right:=Width-1;
        canvas.TextRect(r,(r.Left+r.Right) div 2 - TextWidth div 2, r.Top, s);
      end;
    end;
  end;

  procedure KrAbout.Edit;
  begin
    MessageDlg('KrDFree'+#13+#13+'Version 2.0, 1998'+#13+
               'Minimal Delphi 4.0'+#13+#13+
               'www.iaehv.nl/users/kray/index.html'+#13+
               'kray@iaehv.nl',
                mtInformation, [mbOK], 0);
  end;

  function KrAbout.GetAttributes: TPropertyAttributes;
  begin
    Result := [paMultiSelect, paDialog, paReadOnly];
  end;

  function KrAbout.GetValue: string;
  begin
    Result := '(about)';
  end;
end.
